/*****************************************************************************
SPADE

Spherical particle detection from electron micrographs
Copyright (C) 2000 Anatoly M. Verkhovsky

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

*****************************************************************************/
   

#ifndef _MICROGRAPHBOX
#define _MICROGRAPHBOX 
#include <string.h>

#include <FL/Fl_Box.H>
#include <FL/Fl_Overlay_Window.H>
#include <FL/Fl_Value_Output.H>  
#include <FL/fl_draw.H>
#include <src/fileimage.h>

enum MouseModes{ZoomIn, DrawCircle, DrawBox, MarkArea, None};

class MicroGraphBox : public /*Fl_Box*/Fl_Overlay_Window {
public: 
//	MicroGraphBox (int a, int b, int c, int d): Fl_Box(a,b,c,d){
	MicroGraphBox (int a, int b, int c, int d): Fl_Overlay_Window(a,b,c,d){
		Preview_=false;
		VCentArray=NULL;
		CApath_=NULL;
		Vnum=0;
		ImData_=NULL;
		DrawRect_=false;
		mm_=None;
                end();
	};		// Using standard Fl_Box constructor from fluid
	void draw();
	int handle(int);		        // standard handle method
        void resize(int x,int y, int w, int h); // get larger image to fit window	
        void draw_overlay();                    // abstract function of the super class

        void Preview(const char *, const char *, int);      // Ethan Preview

	void FileImage(Fileimage *a){
		if (a!=fim_) {
			delete(fim_); 
			fim_=a;
		} 
		if (VCentArray!=NULL){ 
			delete(VCentArray);
			VCentArray=NULL;
			CApath_=NULL;
			Preview_=false;
		}
	}   // Set File image for the box

	void UnPreview(){Preview_=false;}       // Turn off preview circles
	void Mmode(MouseModes mm){mm_=mm;}      // Set mouse mode used by control buttons
	MouseModes Mmode(){return mm_;}         // Just in case  
	double Redisplay(Fileimage *, int, int, int, int);
//	void Redisplay(int, int, int, int);     // Redisplay picture
                               // Zoomer functions

	void One2One();				// Set screen resolution to resolution of the image;
        double Shrink2Fit();                    // whole image on screen, returns magnification 

	void PanUp();		// Panner functions
	void PanDown();
	void PanRight();
	void PanLeft();

			    // Output internal data to Value_Out widgets
	void SetWidgetRadius (Fl_Value_Output *wid){Radius_=wid;}
        void SetWidgetBoxSize(Fl_Value_Output *wid){BoxSize_=wid;}
        void SetWidgetZoom   (Fl_Value_Output *wid){Zoom_=wid;}
	void SetCApath (char *path);
//{CApath_=strdup(path);}

	Fileimage *fim_;   //  debug purposes only

private :
	Fl_Image  *flim;    // instead of using label
//        Fileimage *fim_;
        unsigned char *ImData_;  // Image body
	bool Preview_;      // Draw circles over viruses
	int Vnum;	    // Number of viruses detected by ethan
	int *VCentArray;    // Pointer to center coordinates of viruses
	char *CApath_;       // Path to ethan output
	int rad_;           // Radius of a virus used in ethan
	MouseModes mm_;	    // Used in handle to determine what kind of
			    // operation is begin done
	int DnX, DnY;       // coordinates of mousedouwn event
        int DrX, DrY;       // drag coordiantes 
	int ImW_,ImH_;	    // Current picture size
	double Cratio_;        // Current maginication ratio
	bool DrawRect_;     // Whether to draw a rectangle 	
        int LpX_, LpY_, LpW_, LpH_; // Large picture rectangle; 
	Fl_Value_Output *Radius_, *BoxSize_, *Zoom_;
        int CircleX_,CircleY_, CircleRad_;
        int BoxX_,BoxY_, BoxSide_;
};

#endif
