/*****************************************************************************
SPADE

Spherical particle detection from electron micrographs
Copyright (C) 2000 Anatoly M. Verkhovsky

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

*****************************************************************************/
 

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/stat.h>
#include <FL/fl_draw.H>
#include "MicroGraphBox.H"

//===============================================================
void MicroGraphBox::PanUp(){
   if (LpY_!=0){
      if (LpY_-LpH_>0) LpY_-=LpH_;
      else LpY_=0;
      Redisplay(fim_,LpX_,LpY_ ,LpW_,LpH_);
   }
}

//===============================================================
void MicroGraphBox::PanDown(){
int ty=fim_->ySize()-LpH_;
   if (LpY_!=ty){
      if (LpY_+LpH_<=ty) LpY_+=LpH_;
      else LpY_=ty;
      Redisplay(fim_,LpX_,LpY_ ,LpW_,LpH_);
   }
} 

//===============================================================
void MicroGraphBox::PanLeft(){  
   if (LpX_!=0){
      if (LpX_-LpW_>0) LpX_-=LpW_;
      else LpX_=0;
      Redisplay(fim_,LpX_,LpY_,LpW_,LpH_);
   }                                     
}

//===============================================================
void MicroGraphBox::PanRight(){
int tx=fim_->xSize()-LpW_;
   if (LpX_!=tx){
      if (LpX_+LpW_<=tx) LpX_+=LpW_;
      else LpX_=tx;
      Redisplay(fim_,LpX_,LpY_ ,LpW_,LpH_);
   }
} 

//===============================================================
// Simple wrapper to show the whole micrograph

double  MicroGraphBox::Shrink2Fit(){
//   printf("Shrink2fit: %d x %d\n",fim_->xSize(),fim_->ySize());
   return Redisplay(fim_,0,0,fim_->xSize(), fim_->ySize());
}


//===============================================================
// 
 
double MicroGraphBox::Redisplay(Fileimage *fim, int ox, int oy, int tw, int th){
   LpX_=ox; LpY_=oy; LpW_=tw; LpH_=th;
   fim_ = fim;
//   int ratio=0, bwidth=0, 
   int tsize;
   unsigned char *data; 
  fl_cursor(FL_CURSOR_WAIT,FL_BLACK,FL_WHITE);
  Fl::flush();
//  printf("Parameters tw: %d, th: %d\n",tw,th);
  //printf("Hosizontal cratio: %f, vertcal %f\n",((float)th)/(h()-2),((float) tw)/(w()-2)); 

  if((float(th))/(h()-2) > (float (tw))/(w()-2)) Cratio_=(float)th / (h()-2);
     else Cratio_=(float) tw / (w()-2); 
  if (Cratio_<1.0){ 
     printf("Setting Cratio to 1!\n");
     Cratio_= 1.0; 
     ImH_=h()-2; th=ImH_;
     ImW_=w()-2; tw=ImW_;
     if (ImH_>fim->ySize()-oy) ImH_=fim->ySize()-oy;
     if (ImW_>fim->xSize()-ox) ImW_=fim->xSize()-ox;
     data = fim->readForViewing(ox,oy, ImW_,ImH_, ImW_);

  } else {
     ImW_= int (tw/Cratio_);
     ImH_= int (th/Cratio_); 
     data = fim->readForViewing(ox,oy, tw, th, ImW_);
  }

//  if (data==NULL) printf("Boody murder\n");
//  printf("(%d,%d)-(%d,%d) -(%d,%d)\n",ox,oy, tw, th,ImW_,ImH_);

//  printf("Selected cratio: %f \n", Cratio_);
//  printf("Rescale params: ox %d oy %d tw %d th %d ImW_ %d, ImH_ %d\n",
//	ox,oy, tw, th, ImW_, ImH_); 
	// 
//  printf("MGB: image dims %dx%d\n",(int)fim_->xSize(), (int) fim_->ySize());
// printf("Redisplay: before fileimage call: %d x %d\n", fim_->xSize(),fim_->ySize());
//	 printf("red param: %d, %d, %d\n", tw,th,ImW_);


// printf("REdisplay: after fileimage call: %d x %d\n", fim_->xSize(),fim_->ySize());
//  printf ("Image received.\n");
//  printf("New dims: %d %d %d %d\n",ox,oy,tw,th, bwidth);

  if (ImData_) delete (ImData_);
  tsize=ImW_*ImH_; 
//  printf("Size for resize: %d\n",tsize);
  ImData_= new unsigned char [tsize*3];

unsigned char aa;
  for (int i=0;i<tsize;i++){
	aa=data[i];
    ImData_[i*3]=aa;
    ImData_[i*3+1]=data[i];
    ImData_[i*3+2]=data[i];
  }                                                  
  delete(data);
  flim=new Fl_Image(ImData_, ImW_,ImH_) ;
  redraw();
  Fl::flush();
  Fl::check();
  fl_cursor(FL_CURSOR_DEFAULT,FL_BLACK,FL_WHITE);
  return(Cratio_);
}

//=============================================================
//  Overlaoded resize, chnages size of 
//  the image according to space available

void MicroGraphBox::resize(int x,int y, int mw, int mh){
  int ow=w();
  int oh=h();
  float tmpx, tmpy;
//  Fl_Box::resize(x,y,mw,mh);
  Fl_Overlay_Window::resize(x,y,mw,mh);

  tmpx=w(); tmpx/=ow; tmpx*=LpW_;
  tmpy=h(); tmpy/=oh; tmpy*=LpH_;
  if (fim_->xSize() < (int)tmpx + LpX_) tmpx=fim_->xSize()-LpX_;
  if (fim_->ySize() < (int)tmpy + LpY_) tmpy=fim_->ySize()-LpY_;

//  printf("ow: %d, w(): %d\n",ow,w());
//  printf("Calling redisplay: width %d, height %d",LpW_*(w()/ow),LpH_*(h()/oh));
  Redisplay(fim_, LpX_, LpY_, (int)tmpx, (int)tmpy);
  draw();
}

//===========================================================
//  Simple zoom function, setting screen 
//  resolution equal to image resolution

void MicroGraphBox::One2One(){
	if ((LpX_+LpW_/2+w()) > fim_->xSize()){
//		printf("one2one: X oversize detected.\n");
		LpX_=fim_->xSize()-w();
	} 
      	if ((LpY_+LpH_/2+h())>fim_->ySize())
                LpY_=fim_->ySize()-h();         
   int cx=LpX_+LpW_/2;
   int cy=LpY_+LpH_/2;

// If it reaches right end of the screen  
//   if(cx+w()-2>fim_->xSize()) cx=fim_->xSize()-(w()-2);
// if it reaches the bottom of the screen
//   if(cy+w()-2>fim_->ySize()) cy=fim_->xSize()-(h()-2);
                                                  
//   printf("Redisplaying with %d -%d, %d-%d\n",cx-w()/2+1,cy-h()/2+1,w()-2,h()-2);
   Redisplay(fim_, cx-w()/2+1, cy-h()/2+1, w()-2, h()-2);
}


//============================================================
//  overloaded handle function, to trap all the clicks and the drags

int  MicroGraphBox::handle(int event){
int aa,bb;
//	printf("Handle: event: %d\n",event);

   switch (event){
	 case FL_ENTER: 
		fl_cursor(FL_CURSOR_CROSS,FL_BLACK,FL_WHITE);
		Fl::flush();
		break;
	case FL_LEAVE:
		fl_cursor(FL_CURSOR_DEFAULT,FL_BLACK,FL_WHITE);
                Fl::flush();
                break;
         case FL_PUSH:
         DnX=Fl::event_x();
         DnY=Fl::event_y(); 
	 DrawRect_=true;
//         printf("FL_PUSH: detected at (%d,%d)\n",DnX,DnY);
	 break;

      case FL_DRAG:
	 DrX=Fl::event_x();
         DrY=Fl::event_y();
//         printf("FL_Drag: detected at (%d,%d)\n",DrX,DrY); 
	 if (mm_==DrawBox) {
 	   BoxX_=int (LpX_+DnX*Cratio_);
           BoxY_=int (LpY_+DnY*Cratio_);

	   if (DrX-DnX>DrY-DnY) BoxSide_=int((DrY-DnY) *Cratio_);  
		else BoxSide_=int ((DrX-DnX) *Cratio_); 
	   BoxSize_->value(BoxSide_);  
	   BoxSize_->redraw();
	 }
	 if (mm_==DrawCircle){

	     CircleRad_=(int)(((sqrt((DrX-DnX)*(DrX-DnX)+(DrY-DnY)*(DrY-DnY)))/2)*Cratio_);
// WHAT THE ...?
//           Radius_->value(int(((DrX-DnX)/2)*Cratio_));  // show radius in the tab

//	   CircleRad_=(int)(((DrX-DnX)/2)*Cratio_);
	   if (CircleRad_ > 0)	
              Radius_->value((int)CircleRad_);  // show radius in the tab
           Radius_->redraw();
           CircleX_=(int)(LpX_+(DnX+(DrX-DnX)/2)*Cratio_);
           CircleY_=(int)(LpY_+(DnY+(DrY-DnY)/2)*Cratio_);                      
	 }
         redraw_overlay();
	 return 1;
	 break;

      case FL_RELEASE:
//	 printf("FL_RElease caught, cratio:%d !!!\n",Cratio_);
	 if(mm_==ZoomIn){
/*
            printf("Zooming into: (%f,%f)-(%f,%f) DnX:%d Cratio_:%f LpX_:%d\n",
               (DnX)*Cratio_+LpX_, (DnY)*Cratio_+LpY_,
               (DrX-DnX)*Cratio_     , (DrY-DnY)*Cratio_,DnX,Cratio_,LpX_);
            printf(" patch width %f, win width %d\n", (DrX-DnX)*Cratio_, w());
*/
//            printf(" Down (%d,%d) - up (%d,%d)\n", DnX,DnY,DrX,DrY);
//            printf(" bwidth:%d bheight:%d  %d:%d\n", DrX-DnX,DrY-DnY,w(),h());

          if (DrX>ImW_) aa=ImW_; else aa=DrX;
          if (DrY>ImH_) bb=ImH_; else bb=DrY; 

            if (((aa-DnX)*Cratio_<w()) || ((bb-DnY)*Cratio_<h())){ // Zoom is less then 1
//		printf("Zoom correction invoked.\n");
		int a,b;
		if ((fim_->xSize()-((DnX-1)*Cratio_+LpX_))<w())
			a=(int)(fim_->xSize()-((DnX-1)*Cratio_+LpX_));
		else a=w();
		if ((fim_->ySize()-((DnY-1)*Cratio_+LpY_))<h())
			b=(int)(fim_->ySize()-((DnY-1)*Cratio_+LpY_));
		else b=h();
/*
		printf("Offset plus distance x: %f, y:%f, a: %d b: %d\n",
		(DnX-1)*Cratio_+LpX_+a, (DnY-1)*Cratio_+LpY_+b,a,b);
               Redisplay(fim_,
      		  (DnX-1)*Cratio_+LpX_, (DnY-1)*Cratio_+LpY_,
		  a, b);
*/
            }else                                // Zoom Greater or equal 1
               Redisplay(fim_,
 		 (DnX/*-x()*/-1)*Cratio_+LpX_, (DnY/*-y()*/-1)*Cratio_+LpY_,
		 (aa-DnX)*Cratio_     , (bb-DnY)*Cratio_);
	    DrawRect_=false;
	    Zoom_->value(Cratio_);
	    Zoom_->redraw();
            redraw();
	 }
//	 redraw();
        redraw_overlay();
	return 1;
	break;
      default:
	return 0;
   }
   return 1;  // this is just for compiler to be happy
//   return Fl_Box::handle(event);
//printf("After handle xsize: %d\n",fim_->xSize()); 
} 

//============================================================
//  Overloaded draw

void MicroGraphBox::draw(){
//printf("Before Draw xsize: %d\n",fim_->xSize()); 
   Fl_Overlay_Window::draw();
   fl_draw_image(ImData_,/*x()+1*/1,/*y()+1*/1,ImW_,ImH_);
//printf("After Draw xsize: %d\n",fim_->xSize());  
}                      

//=====================================================================
void MicroGraphBox::draw_overlay(){
   int n=int(rad_/Cratio_);
   int a,b;
// printf("Before overlay xsize: %d\n",fim_->xSize());
//   printf("Draw overlay is called! preview: %d\n", Preview_);
   fl_clip(1,1,ImW_,ImH_);
   if(Preview_){ 
//	printf("ethan preview: %d!\n",Vnum);
      fl_color(FL_RED);
      for(int i=0;i<Vnum;i++){
	 fl_circle(/*x()+*/(VCentArray[i*2]-LpX_)/Cratio_,
                   /*y()+*/(VCentArray[i*2+1]-LpY_)/Cratio_, n);
         fl_circle(/*x()+*/(VCentArray[i*2]-LpX_)/Cratio_,
                   /*y()+*/(VCentArray[i*2+1]-LpY_)/Cratio_, n-1);
      }
   }
   if (DrawRect_)

   switch (mm_){
      case ZoomIn:
	   if (DrX>ImW_) a=ImW_; else a=DrX;
	   if (DrY>ImH_) b=ImH_; else b=DrY;
	
           fl_color(FL_GREEN); 
           fl_rect(DnX, DnY, a-DnX, b-DnY);
           fl_rect(DnX+1, DnY+1, a-DnX-2, b-DnY-2);
	break;
      case DrawCircle:
//        if (DrawRect_){
           fl_color(FL_GREEN);
	   fl_circle((CircleX_-LpX_)/Cratio_,(CircleY_-LpY_)/Cratio_, CircleRad_/Cratio_);
	   fl_circle((CircleX_-LpX_)/Cratio_,(CircleY_-LpY_)/Cratio_,CircleRad_/Cratio_+1);
//	}
	break;
      case DrawBox:
//	if (DrawRect_){
           fl_color(FL_GREEN); 
	   fl_rect(int ((BoxX_-LpX_)/Cratio_),int ((BoxY_-LpY_)/Cratio_),
		int(BoxSide_/Cratio_),int(BoxSide_/Cratio_));
//	}
	break;
      default:
	;            // to handle modes that are not implemented yet
   }
   fl_pop_clip();
}


//=====================================================================
//  Run ethan in preview mode,
//  code mostly copied from old CASE

void MicroGraphBox::Preview(const char *datapath, const char *ethancfg,
int rad){
   FILE *fp;
//printf("CA path: %s\n",CApath_);

   if (VCentArray==NULL){   // We we don't have the list of centers yet
      char ethancmd[500];
      char wrk[500];      
      if (CApath_==NULL){	
         sprintf(ethancmd, "ethan %d %s /tmp/ethan.tmp %s", 
		rad, datapath, ethancfg);
         getcwd(wrk, 200);               // save current working directory 
//         chdir(ethanpath);
         if (system(ethancmd) == -1) printf("Running ethan failed\n");
//	   printf("Ethan finished!\n");  //run ethan
//         chdir(wrk);                     //return to working directory
//-------------------------------------------------------------------
        fp = fopen("/tmp/ethan.tmp", "r");    // open temporary ethan output file
	chmod("/tmp/ethan.tmp", S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH); 
     }else {
	fp=fopen(CApath_, "r");
//	printf("Opening %s\n",CApath_);
      } 
     int i=0,i1,i2;
      Vnum=0;
      rad_=rad;
      //printf("Before first read!\n");
      do{                                   // First run, just go throu
         fscanf(fp, "%d%d\n", &i1, &i2);    // and count all records
         Vnum++;     
      }while(!feof(fp));

//   printf("After first read %d records!\n",Vnum);
      rewind(fp);                           
      VCentArray= new int[Vnum*2];
//(int *)malloc(Vnum*2*sizeof(int));  //Get meory for the list
      do{                                    // Second run, collecting data 
         fscanf(fp, "%d%d\n", VCentArray+i, VCentArray+i+1);
//      printf("%d[x= %d, y = %d]     ",i,VCentArray[i], VCentArray[i+1]);
         i+=2;
      }while(!feof(fp));
      fclose(fp);
   }
   Preview_ = true;
}



void MicroGraphBox::SetCApath (char *path){CApath_=strdup(path);Preview_=true;}
