/*****************************************************************************
SPADE

Spherical particle detection from electron micrographs
Copyright (C) 2000 Anatoly M. Verkhovsky

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

*****************************************************************************/




//===============================================================
//  here are the definitions of tags used in a project file
//  current size of the record is 256... maybe should be reduced to
//  128 ???
//
//  It was decided to increase record size to 512 in order
//  for input files to save complete path  28.10.99


#define PROJECT_TAG  0
#define PROJECT_PATH 1
#define ETHAN_PATH   2
#define MRC_PATH     3
#define GROUP        4
#define INPUT_FILE   5
#define OUTPUT_FILE  6
#define ETHAN_CFG    7
#define EXTRASP       256

#define PATHSIZE 256
#define NAMESIZE 30

#define MAGICSTR "This file was created by SPADE."

typedef struct PrjID{
   unsigned char tag;             // 1
   short    int  version;         // 2    
            char Auth_Str[251];
	    char rsrv[EXTRASP];
}PrjID;

typedef struct EthanCFG{
   unsigned char tag; 
	char filename[PATHSIZE];
	float EthanParHeight;
	float EthanParWidth;
	float EthanParRWidth;
	float EthanParDist;
	float EthanParReduct;
	char EthanParResIm;
	char EthanParRefine;
        char EthanTestSector;
        char EthanTestHeight;
	char EthanTestDistance;
        char rsrv[224];
}EthanCFG;

typedef struct PrjPath{
   unsigned char tag;         // 1
   short    int  fileno;      // 2
            char path[PATHSIZE];   // 253
            char rsrv[EXTRASP-5]; 
}PrjPath;

typedef struct PrjGroup{
   unsigned char tag;            // 1   tag to distingish the type of record
   short    int  groupno;        // 2   Number of the group starting from 1
   short    int  radius;         // 2   Virus radius specified for this group 
   short    int  box;            // 2   Box size for this group in pixels
            char processed;      // 1   Processed flag should be boolean
            char grname[NAMESIZE]; // 241 Name of the group
            char rsrv[EXTRASP+211];      //
   struct PrjGroup *next;        // 4   This space is reserved for a pointer
}PrjGroup;                       //     to build linked lists fo groups  

typedef struct PrjInFile{
   unsigned char tag;                 // 1   tag
   short    int  infileno;            // 2   Number of the file in that group starting from 1 
            char infname [NAMESIZE];  // 30  filename
	    char infpath [PATHSIZE];  // 256 path to input file	
	    char grname  [NAMESIZE];  // 30  groupname
	    char rsrv    [185];       // 
   struct PrjInFile *next;            // 4   next pointer
}PrjInFile;


typedef struct PrjOutFile{
   unsigned char tag;                   // 1
   short    int  outfileno;             // 2   number of the output file
	    char fordel;                // 1   whether sheduled for deletion	
            char infname [NAMESIZE];    // 30 
            char grname  [NAMESIZE];    // 30                     
            char oufname [NAMESIZE];    // 30 
	    char rsrv    [EXTRASP+154]; // 154	
   struct PrjOutFile * next;            // 4   link to next
}PrjOutFile;
