/*****************************************************************************
SPADE

Spherical particle detection from electron micrographs

Copyright (C )1999 T. Kaituri, E. Kallioniemi, J. Paaso,
M. Ters and J. Virtanen

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

*****************************************************************************/
                                                                              

#include <stdio.h>
#include <iostream.h>
#include <fstream.h>
#include <sys/stat.h>
//#include <png.h>
//#include <zlib.h>
#include "globals.h"

#include "fileimage.h"
#include "image8bi.h"


/*
 * Method:   Fileimage(const char *filename)
 *           filename is the name for file to be "attached" for
 *           the object -> it is not used until further method-calls
 *
 * Purpose:  Create new object
 * Returns:  Nothing
 * Synopsis: Constructor makes a copy of the filaname passed as
 *           parameter and makes a check() for the specified file.
 *           check() may cause exception if the specified file 
 *           cannot be read.
 */


Fileimage::Fileimage(const char *filename) 
{
//     char *tmp = new char [strlen(filename) + 1];
//     sprintf(tmp, "%s", filename);
//     _name = tmp;
     int a;
     _name = strdup(filename);
     _width    = 0;  _height   = 0;
     _sections = 0;  _mode     = 0;
     
     if((a=check())<1){ 
//	printf("Did not pass check, a=%d \n",a);
	check();
	throw FileReadFailed("Problems opening file!");
      }
     return;
}

/*
 * Method:   saveImage(const char *filename, unsigned char *data,
 *           int sizex, int sizey, int format)   
 *           static method, use like this: Fileimage::saveImage(...
 * Purpose:  For saving files in either png or mrc -format
 * Returns:  void
 * Synopsis: 8-bit grayscale raw data may be saved as a file using
 *           this method. Format may be either png (file_png) or
 *           mrc (file_mrc). 
 *
 */

// this method is commented out in order to avoid use of png lib 
void Fileimage::saveImage (const char *filename, unsigned char *data, 
				  int sizex, int sizey, int format)
{
/*
     if (format == file_png) {

	  FILE *fd = fopen(filename, "w"); 

	  if (!fd) {      // generate exception if 
	       char *retname = new char[strlen(filename) + 1];
	       sprintf(retname, "%s", filename);
	       printf("avaus %s eponnistui\n", retname);
	       throw FileWriteFailed(filename);
	  }

	  png_structp png_ptr = png_create_write_struct
	       (PNG_LIBPNG_VER_STRING, NULL,  NULL, NULL);

	  if (!png_ptr)
	       return;
     
	  png_infop info_ptr = png_create_info_struct(png_ptr);

	  // Used for "headers"
	  
	  if (!info_ptr)
	  {
	       png_destroy_write_struct(&png_ptr,
					(png_infopp)NULL);
	       return;
	  }

	  if (setjmp(png_ptr->jmpbuf))
	  {
	       // If we get here, we had a problem reading the file 
	       fclose(fd);
	       png_destroy_write_struct(&png_ptr,  (png_infopp)NULL);
	       return;
	  }

	  png_init_io (png_ptr, fd);
	  png_set_IHDR (png_ptr, info_ptr, sizex, sizey, 8, PNG_COLOR_TYPE_GRAY, 
			PNG_INTERLACE_NONE,  PNG_COMPRESSION_TYPE_BASE, 
			PNG_FILTER_TYPE_BASE);

	  png_write_info (png_ptr, info_ptr);

	  for (int i = 0; i < sizey; i++) 
	       png_write_row (png_ptr, &data [i * sizex]);
          
	  png_write_end(png_ptr, info_ptr);

	  png_destroy_write_struct(&png_ptr, &info_ptr);

	  fclose(fd);
     } // if (format == file_png)

     else if (format == file_mrc) {

	  ofstream outfile(filename, ios::binary | ios::out);
	  
	  if (!outfile) {
	       char *retname = new char[strlen(filename) + 1];
	       sprintf(retname, "%s", filename);
	       outfile.close();
	       throw FileWriteFailed(retname);
	  }
	  
	  int i = 0;

	  outfile.write((char *) &sizex, 4);
	  outfile.write((char *) &sizey, 4);
	  outfile.write((char *) &i, 4);
	  outfile.write((char *) &i, 4);
	  
 
	  char *garbage = new char[1008];               // something sensible?
	  outfile.write((char *) garbage, 1008);        // Write headers 
	  outfile.write((char *) data, sizex * sizey);  // Write data
	  
	  outfile.close();
     } // if (format==file_mrc)
*/
     return;
}

Fileimage::~Fileimage () 
{


}

/*
 * Method:   readForProcessing()
 *
 * Purpose:  The data from the associated file is loaded 
 * Returns:  (currently) pointer to a Image8Bit -object
 * Synopsis: If the data from the associated file is needed,
 *           no scaling has to be done for the data, this method
 *           should be used. Also if 16-bit (or 12-bit) handling is 
 *           to be implemented some day, data may be read with this
 *           method (return Image8Bit or Image16Bit) 
 */

Image *Fileimage::readForProcessing() {

          
     check();
     
     unsigned char *data;

     ifstream infile (_name, ios::binary | ios::in);    // Luodaan input-fstream olio

     if (!infile) {
	  char *retname = new char[strlen(_name) + 1];
	  sprintf(retname, "%s", _name);
	  throw FileReadFailed(retname);
     }

     data = new unsigned char [1 * _width * _height];  // 8bit grayscale -data

     unsigned char *garbage = new unsigned char [_dataOfst];

     infile.read ((char *) garbage, _dataOfst);           // headers (add some check here)
     infile.read ((char *) data, _width * _height);    // Read data

     // puuh, pitis pari muuttujaa lukea sielt viel lis

     Image8Bit *retu = new Image8Bit (_width, _height, data, 255);
     
     return (retu);
}

/*
 * Method:   bool dataAmountOk()
 *
 * Purpose:  Checks if corresponding file has width * height * mode
 *           of data
 * Returns:  true if ok, else false
 * Synopsis: 
 */

bool Fileimage::dataAmountOk(ifstream &mrc)
{
     int enough, toomuch;       // check for amount for data

     // Move the filepointer to the end of the file

     mrc.seekg (0);
     mrc.seekg (_dataOfst + (_width * _height), ios::cur);
     
     if (mrc.peek() != EOF) enough = 0;    // peek for eof
     else                   enough = 1;

     mrc.seekg (1, ios::cur);   // go past the last byte

     if (mrc.peek() == EOF) toomuch = 0;
     else
     toomuch = 1;

     if (enough && !toomuch) {
	  mrc.close();
	  return (true);
     }     

     else {
	  // Try to change endianess

	  _width = chEndian (_width); _height = chEndian (_height);

	  mrc.seekg (0);
	  mrc.seekg (_dataOfst + (_width * _height), ios::cur);
	  
	  if (mrc.peek() != EOF) enough = 0;
	  else                   enough = 1;
	  
	  mrc.seekg (1, ios::cur);
	  
	  if (mrc.peek() == EOF) toomuch = 0;
	  else                   toomuch = 1;
	  
	  if (enough && !toomuch) {
	       mrc.close();
	       return (true);
	  }
	  else {                 // If both failed throw exception
	       mrc.close();
	       return (false);
	  }
     }
          
     return (false);
}

/*
 * Method:   int check()
 *
 * Purpose:  reads
 * Returns:  true if ok, else false
 * Synopsis: 
 */

int Fileimage::check() {
   struct MRC_header head;
   FILE *fp;
   struct stat mstat;
   char s[5],c;
   int tmp;
//========================================================
//   MRC data file
     if (!(_width == 0 && _height == 0))  return (1);
     if (stat(_name, &mstat)==-1) return 0;
     if ((fp=fopen(_name,"rb"))==NULL){
	 perror("fileimage");
	 return 0;
     }
     if (strcmp("mrc", &(_name[strlen(_name)-3]) )==0){

        fread(&head, sizeof(struct MRC_header),1,fp);
        _height = head.height;   //sections and mode are not used
        _width  = head.width;
        _dataOfst = 1024;
        if ((_dataOfst + _width*_height) != (unsigned)mstat.st_size) {
           _width=chEndian(_width); _height=chEndian(_height);
           if ((_dataOfst + _width*_height) != (unsigned) mstat.st_size)
return 0;
        }
     }
//========================================================
//   PGM data file
     if (strcmp("pgm", &(_name[strlen(_name)-3]))==0){
        fgets(s,4,fp);
        if (strcmp(s,"P5\n")!=0) return 0;
        while ((c=fgetc(fp))=='#')
          while ((fgetc(fp))!='\n');
        ungetc(c,fp);
        fscanf(fp,"%d %d %d",&_width, &_height,&tmp);
        fgetc(fp);
        _dataOfst = ftell(fp);
     }
     fclose(fp);
/*
     ifstream mrc (_name, ios::binary | ios::in);
     if (!mrc) {
//	  char *retname = new char [sizeof(_name) + 1];
//	  sprintf(retname, "%s", _name);
//	  printf("fileimage: file name \"%s\" \n",_name);
//	  throw FileReadFailed(_name);

	  mrc.close();
	  printf("Failed to open %s.\n", _name);
          return 0;
     }
     
     // Read the important header information

     mrc.read ((char *) &_width, 4);
     mrc.read ((char *) &_height, 4);
//printf("fileimage: Read mrc dimensions %fx%f \n", _width, _height); 
     mrc.read ((char *) &_sections, 4);  // why do this if it's not used anywhere???
     mrc.read ((char *) &_mode, 4);      // again, why?
     

     if (_width == 0 || _height == 0) {
	  char *retname = new char [sizeof(_name) + 1];
	  sprintf(retname, "%s", _name);
	  mrc.close();
	  throw FileReadFailed("homo");
     }

     if (dataAmountOk(mrc)) {
	  mrc.close ();
	  return (1);
     }
     else mrc.close();

//	  char *retname = new char [sizeof(_name) + 1];
//	  sprintf(retname, "%s", _name);
//	  throw FileReadFailed(retname);
//     }
*/
     return 1;
}

/*
 *  Changes the endianess of a unsigned integer
 */ 

unsigned Fileimage::chEndian (unsigned b)
{
     return ((b & 0xFF000000) >> 24) | ((b & 0x00FF0000) >> 8) |
	  ((b & 0x0000FF00) << 8) | ((b & 0x000000FF) << 24);
}

/*
 * Method:   unsigned char *readForViewing(int realx, int realy,
 *           int realdimx, int realdimy, int scaledx)
 *
 * Purpose:  
 * Returns:  
 * Synopsis: 
 *           
 *           
 *           
 *           
 */

unsigned char *Fileimage::readForViewing(int realx, int realy, int realdimx,
					 int realdimy, int scaledx) {

     check();
//     printf("fileimage:  Dimensions after check %dx%d \n",(int) _width, (int) _height);

     if ((realx + realdimx) > (int) _width) {
	  std::cout << "cse:fileimage: Internal software error [range problem]" << endl;
//	  printf("realx %d, realdimx %d, width %d\n",realx,realdimx,(int) _width);
	  exit (0);
     }

     if (scaledx > realdimx) {
	  std::cout << "cse:fileimage: Internal software error [scaling problem]" <<
endl;
	  exit (0);
     }
	  
     // If file contains "16"-bit data, convert it to 8bit

     ifstream infile (_name, ios::binary | ios::in);

     infile.seekg (_dataOfst, ios::cur);            // Skip headers
     infile.seekg ((realy * _width) + realx,    // Locate starting coordinate
		   ios::cur);   
     
//     int scaledy = (int) ((double) (scaledx * realdimy) / (double) realdimx); 
//    Added one, just in case...
    int scaledy = (int) ((double) (scaledx * realdimy) / (double) realdimx)+1;
     int avgdimX = realdimx / (int) (realdimx / scaledx);   

     unsigned char *final = 
	  new unsigned char [scaledx * scaledy];  // Final scaled data
//     printf("fileimage: Output size %dx%d\n", scaledx , scaledy);
     int avgdim = realdimx / (int) (realdimx / scaledx);   

     // Two lines are needed for interpolation

     int lineANum = -1;     int lineBNum = -1;            
     int realNum = -1;   // Contains the real line number

     unsigned char *lineA = new unsigned char [avgdim];
     unsigned char *lineB = new unsigned char [avgdim];

     // define the steps for interpolation

     double Yscale = ((double) realdimy / (double) scaledy) / (realdimx / scaledx);
     double Xscale = (double) avgdimX  / (double) scaledx;

     double SPRow, RowDelta, SPCol, ColDelta;
     int SPRowNum, SPColNum;

     // For every line in the destination image (final image)

     for (int destRow = 0; destRow < scaledy; destRow++) {
	  SPRow = Yscale * (double) destRow;
	  SPRowNum = (int) SPRow;
	  RowDelta = SPRow - SPRowNum;

	  // SPRowNum tells what line(s) has to be read for interpolation 

	  // Handle interpolation lines:

	  if (lineANum == SPRowNum) {
	       if (lineBNum != (SPRowNum + 1)) {
		    ++ realNum;
		    readLine (lineB, infile, realdimx, scaledx, _width);
		    lineBNum = realNum;
	       } 
	  }
	  else if (lineBNum == SPRowNum) {
	       if (lineANum != (SPRowNum + 1)) {
		    ++ realNum;
		    readLine (lineA, infile, realdimx, scaledx, _width);
		    lineANum = realNum;
	       }
	  }
	  else {
	       ++ realNum;
	       readLine (lineA, infile, realdimx, scaledx, _width);
	       lineANum = realNum;
	       ++ realNum;
	       readLine (lineB, infile, realdimx, scaledx, _width);
	       lineBNum = realNum;
	  }

	  // For every column in the destination image

	  for (int destCol = 0; destCol < scaledx; destCol++) {
	       SPCol = Xscale * (double) destCol;
	       SPColNum = (int) SPCol;
	       ColDelta = SPCol - SPColNum;

	       if (lineANum == SPRowNum)
		    final [(destRow * scaledx) + destCol] = // (int) PixValue;
			 pixelValue (SPColNum, lineA, lineB, ColDelta, RowDelta);
	       else
		    final [(destRow * scaledx) + destCol] = // (int) PixValue;
			 pixelValue (SPColNum, lineB, lineA, ColDelta, RowDelta);
	  }
     }

     delete [] lineA;
     delete [] lineB;

//     Image8Bit *returnValue = new Image8Bit (scaledx, scaledy, final, 255); 
     
     // No need for returning image -object, instead returning raw data

     return (final);
}


unsigned char *Fileimage::readForViewing(int realx, int realy, int realdimx,
					 int realdimy, int scaledx, int &y) {

     check();

     if ((realx + realdimx) > (int) _width) {
	  printf("l yrit lataa enempi ko kuvaa on\n");
	  exit (0);
     }

     if (scaledx > realdimx) {
	  printf("Funktio ei osaa skaalata suuremmaksi\n");
	  exit (0);
     }
	  
     // If file contains "16"-bit data, convert it to 8bit

     ifstream infile (_name, ios::binary | ios::in);

     infile.seekg (_dataOfst, ios::cur);                      // Skip headers
     infile.seekg ((realy * _width) + realx, ios::cur);   // Locate starting coordinate

     if (scaledx > realdimx) {
//	  scaledx = realdimx;
     }

     int scaledy = (int) ((double) (scaledx * realdimy) / (double) realdimx); 

     int avgdimX = realdimx / (int) (realdimx / scaledx);   

     unsigned char *final = new unsigned char [scaledx * scaledy];  // Final scaled data

     y = scaledy;

     int lineANum = -1;     int lineBNum = -1;            
     int realNum = -1;                                  // Contains the real line number

     int avgdim = realdimx / (int) (realdimx / scaledx);   

     unsigned char *lineA = new unsigned char [avgdim];
     unsigned char *lineB = new unsigned char [avgdim];

     // "Optimization" follows:

     double Yscale = ((double) realdimy / (double) scaledy) / (realdimx / scaledx);
     double Xscale = (double) avgdimX  / (double) scaledx;

     double SPRow, RowDelta, SPCol, ColDelta;
     int SPRowNum, SPColNum;


     // For every line in the destination image (final image)

     for (int destRow = 0; destRow < scaledy; destRow++) {
	  SPRow = Yscale * (double) destRow;
	  SPRowNum = (int) SPRow;
	  RowDelta = SPRow - SPRowNum;

	  // SPRowNum tells what line(s) has to be read for interpolation 

	  // Handle interpolation lines:

	  if (lineANum == SPRowNum) {
	       if (lineBNum != (SPRowNum + 1)) {
		    ++ realNum;
		    readLine (lineB, infile, realdimx, scaledx, _width);
		    lineBNum = realNum;
	       } 
	  }
	  else if (lineBNum == SPRowNum) {
	       if (lineANum != (SPRowNum + 1)) {
		    ++ realNum;
		    readLine (lineA, infile, realdimx, scaledx, _width);
		    lineANum = realNum;
	       }
	  }
	  else {
	       ++ realNum;
	       readLine (lineA, infile, realdimx, scaledx, _width);
	       lineANum = realNum;
	       ++ realNum;
	       readLine (lineB, infile, realdimx, scaledx, _width);
	       lineBNum = realNum;
	  }

	  // For every column in the destination image

	  for (int destCol = 0; destCol < scaledx; destCol++) {
	       SPCol = Xscale * (double) destCol;
	       SPColNum = (int) SPCol;
	       ColDelta = SPCol - SPColNum;

	       if (lineANum == SPRowNum)
		    final [(destRow * scaledx) + destCol] = // (int) PixValue;
			 pixelValue (SPColNum, lineA, lineB, ColDelta, RowDelta);
	       else
		    final [(destRow * scaledx) + destCol] = // (int) PixValue;
			 pixelValue (SPColNum, lineB, lineA, ColDelta, RowDelta);
	  }
     }

     delete [] lineA;
     delete [] lineB;

//     Image8Bit *returnValue = new Image8Bit (scaledx, scaledy, final, 255); 
     

     return (final);
}


/* 
 * Function interpolates pixelvalue from four pixels in two lines
 *
 */

int Fileimage::pixelValue (int ColNum, unsigned char *lineA, unsigned char *lineB,
			   double ColDelta, double RowDelta)
{
     int PtA = lineA [ColNum];
     int PtB = lineB [ColNum];
     int PtC = lineA [ColNum + 1];
     int PtD = lineB [ColNum + 1];
     
     double ContrAB = ColDelta * ((double) PtB - PtA) + PtA;
     double ContrCD = ColDelta * ((double) PtD - PtC) + PtC;
     
     double PixValue = ContrAB + (ContrCD - ContrAB) * RowDelta;
     
     return ((int) PixValue);
}


/*
 * Method: readLine
 * Purpose:
 *    This function reads "few" lines and scales them to size
 *    approximately destination image size
 * Retuns: void
 *
 */

void Fileimage::readLine(unsigned char *dest, ifstream &infile, 
				       int realdimx, int scaledx, int width)
{
     int sum, position;  
     unsigned char *rowData = new unsigned char [realdimx]; 
     
     int avgdim = realdimx / (int) (realdimx / scaledx);   // How long is the scaled line
     int scale = realdimx / scaledx;                       // What's the scale factor

     unsigned char *lineAverage =                          // One line is averaged to this
	  new unsigned char [avgdim];

     // Perform rough scaling 
          
     for (int dy = 0; dy < avgdim; dy ++)
	  dest [dy] = 0;

     for (int dx = 0; dx < scale; dx++) {                // Read scale lines
	  infile.read ((char *) rowData, realdimx);      // Read one line
	  infile.seekg (width - realdimx, ios::cur);	 // Skip rest of the data 

	  for (int dy = 0; dy < avgdim; dy++) {          // For every pixel in avg
	       sum = 0;
	       position = dy * scale;                    // Locate position 
	       for (int x = 0; x < scale; x++)           // Calculate average "around" position
		    sum += rowData [position + x];
	       sum /= scale;
	       lineAverage [dy] = sum;
	  }
	  for (int dy = 0; dy < avgdim; dy++) {          // Compute everything into 
	       int value = lineAverage [dy] / scale;     // one line
	       dest [dy] += (unsigned char) value;
	  }

     }  

     delete [] lineAverage;
     delete [] rowData;
     
     return;
}




