/*****************************************************************************
SPADE

Spherical particle detection from electron micrographs

Copyright (C )1999 T. Kaituri, E. Kallioniemi, J. Paaso,
M. Ters and J. Virtanen

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

*****************************************************************************/

/*
 * globals.h -- Enumerations and exceptions defined in the global
 *              namespace.  Also contains generic min(...) and 
 *              max(...) functions, since they are invaluable
 *              much of the time.
 *
 * version 1.0 by Mikko Teras, May 1999, CASE project, U of Helsinki
 */

#ifndef _CASE_GLOBAL_NAMES_
#define _CASE_GLOBAL_NAMES_
#include <string.h>

/* Global typedefs needed by several different compilation units */

enum actionRq {exitProgram, openMicrographWin, openMosaicWin};
enum ImageOps {thsc,thpc};

/* Exceptions used by several different compilation units */

class OutOfMemory {};                                 //Dummy exception class

class FileReadFailed {                                //For read errors:
public:                                               
    FileReadFailed (const char *s = "") {             //Exception takes a string
	desc = new char[strlen(s)+1];                 //containing the file name
        strcpy(desc, s);
    }
    char * what () { return desc; }                   //what gets the file name
    FileReadFailed (const FileReadFailed &rhs) {      //Deep copy constructor,
        desc = new char[sizeof(rhs.desc)];                    //needed by
throw
        strcpy(desc, rhs.desc);
    }
    ~FileReadFailed () { delete[] desc; }             //Destructor, free memory
private:
    char *desc;
};

class FileWriteFailed {                               //For write errors:
public:                                               //Exception takes a string
    FileWriteFailed (const char *s = "") {            //containing the file name
        desc = new char[strlen(s)+1]; 
        strcpy(desc, s); 
    }
    FileWriteFailed (const FileWriteFailed &rhs) {
    	desc = new char[strlen(rhs.desc)+1];
    	strcpy(desc, rhs.desc);
    }
    char * what () { return desc; }                   //what gets the file name
    ~FileWriteFailed () { delete[] desc; }            //Destructor, free memory
private:
    char *desc;
};

/* Simple template functions for getting min and max of two elements.    */
/* Defined here since they are needed a lot in several different places. */

template<class T> inline T min (T op1, T op2) {
    return op1<=op2 ? op1 : op2;
}

template<class T> inline T max (T op1, T op2) {
    return op1>=op2 ? op1 : op2;
}

#endif
