/*****************************************************************************
SPADE

Spherical particle detection from electron micrographs

Copyright (C) 1999 T. Kaituri, E. Kallioniemi, J. Paaso,
M. Ters and J. Virtanen

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

*****************************************************************************/
    
/*
* CASE - Software engineering project
* University of Helsinki, Department of Computer Science
* Spring 1999
*
*
* File:     image8bi.h
* Purpose:  Header (definition) for Image8Bit - class.
* Use:      This is header is linked with all the modules using
*           Image8Bit - class objects.
*
*
* Synopsis: This is direct derivation of the Image-class. Whole
*           interface is exactly same. Only constructor and destructor
*           are subclass specific. Private data includes definition for
*           histogram and imagedata tables.
*
*
* Author:   Esa Kallioniemi
* Date:     15.5.1999
*
*/

#ifndef BIMAGE_H_
#define BIMAGE_H_

#include "image.h"

class Image8Bit : public Image {

    public:
        Image8Bit(int xsize, int ysize, unsigned char* data,
                  int maxIntensity);
        ~Image8Bit();

        int     xSize();
        int     ySize();
        imageType type();
        int     physSize();
        void*   physData();
        void*   copyData();
        int     standardDeviation();
        int 	average();
        Image*  lineProfile(int xstart, int ystart, int length, int heigth);
        Image*  histogramProfile(int size);
        int     valueAt(int x, int y);

                
        Image*  powerSpectrum2D();
        Image*  rotationalAverage(float step);
		
        Image*  scale(int destXSize, int destYSize);

        int     doPadding(int xSize, int ySize);
        Image * cutArea(int upperLeftX, int upperLeftY, int xDim, int yDim);
		Image* 	equalize();

        void    setMask(Mask* msk);
        int     centerHistogram();
        int     transformHistogramStatisticalCut(int stdDevMul);
        int     transformHistogramPercentCut(int lprc, int rprc);
        int     blendWithMask();

        int     compositeTransform(int stdDevMul);
        int     compositeTransform(int lprc, int rprc);
        
        protected:
            void* calculateHistogram();


    private:
        unsigned char *imageData;   // Pointer to the image data
        unsigned int *histogram;    // Pointer to the histogram
};
#endif

