/*****************************************************************************
SPADE

Spherical particle detection from electron micrographs

Copyright (C) 1999 T. Kaituri, E. Kallioniemi, J. Paaso,
M. Ters and J. Virtanen

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

*****************************************************************************/
    
#include "imageop.h"


Operation::Operation() 
{
     
     // No operations on particles

     mask    = false;
     
     blend   = false;
     center  = false;
     stretch = false;

     return;
}

int Operation::getmaskin()
{
     return maskin;
}

int Operation::getmaskout()
{
     return maskout;
}

int Operation::getperA()
{
     return perA;
}

int Operation::getperB()
{
     return perB;
}

int Operation::getkh()
{
     return keskihajonta;
}



Operation::Operation(int rmaskin, int rmaskout, bool rblend, bool rcenter, 
		     bool rstretch, bool rpercent, int ra, int rb, int kh) 
{


     maskin  = rmaskin;
     maskout = rmaskout;

     // ^^^^^^^^^ <- Create mask here?

     blend =  rblend;
     center = rcenter;

     if (rstretch) {
	  if (rpercent) {
	       perA = ra; perB = rb;
	       kh = 0;
	  }
	  else {
	       keskihajonta = kh;
	       perA = 0; perB = 0;
	  }
     }
     else
	  stretch = false;

     return;
}


void Operation::setmask(bool maskbool, int in, int out)
{
     if (maskbool) {
	  mask = true;
	  maskin = in;
	  maskout = out;
     }
     else {
	  blend = false;
	  center = false;
	  stretch = false;
     }
}


void Operation::setblend(bool ublend)
{
     blend = ublend;

     return;
}

void Operation::setcenter(bool ucenter)
{
     center = ucenter;

     return;
}

void Operation::setstretch(bool ustretch, int a, int b)
{
     if (ustretch) {
	  stretch = true;
	  percent = true;
	  perA = a;
	  perB = b;
     }
     else 
	  stretch = false;
     

}

void Operation::setstretch(bool ustretch, int koohoo)
{
     if (ustretch) {
	  stretch = true;
	  percent = false;
	  keskihajonta = koohoo;
     }
     else 
	  stretch = false;
}

bool Operation::useblend()
{
     
     if (mask)
	  return (blend);
     else
	  return (false);
}

bool Operation::usecenter()
{

     if (mask)
	  return (center);
     else
	  return (false);
}

bool Operation::usestretch()
{

     if (mask)
	  return (stretch);
     else
	  return (false);
}

bool Operation::usepercent()
{

     if (mask) {
	  if (stretch)
	       return (percent);
	  else
	       return (false);
     }
     else
	  return (false);
}
	
Operation::manipulation Operation::useoperations()
{

     if (blend && center && stretch)
	  return (all);
     else if (blend && stretch && !center)
	  return (blendstretch);
     else if (blend && !stretch && center)
	  return (blendcenter);
     else if (blend && !stretch && !center)
	  return (oblend);
     else if (!blend && stretch && center)
	  return (stretchcenter);
     else if (!blend && stretch && !center)
	  return (ostretch);
     else if (!blend && !stretch && center)
	  return (ostretch);
     else 
	  return (none);

     return (none);
}


