/*****************************************************************************
SPADE

Spherical particle detection from electron micrographs

Copyright (C) 1999 T. Kaituri, E. Kallioniemi, J. Paaso,
M. Ters and J. Virtanen

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

*****************************************************************************/
    
/*
* CASE - Software engineering project
* University of Helsinki, Department of Computer Science
* Spring 1999
*
*
* File:     mask.h
* Purpose:  Header for Mask-class. Includes interface and class data
*           definitions.
* Use:      This file is linked with all modules using Mask-class
*           objects.
*
*
* Synopsis: Mask-class is data object for image mask used in Image-class
*           hierarcy. Mask consist of definition for inner circle
*           (visible area) and blend edge (refered as outer edge)
*           surrounding inner circle. Outer and inner circles are
*           stored in form of scanline edge tables. Tables are generated
*           using circle algorithm. Blend is constructed using
*           alpha-channel and cosine function for alpha distribution.
*           Intensity of alpha grows outwards, so that background has
*           stronger alpha intensity ( increasing presence).
*
*
*
* Author:   Esa Kallioniemi
* Date:     15.5.1999
*
*/

#ifndef MASK_H_
#define MASK_H_

//============ irix glitch
#ifndef PI
#define PI        3.14159265358979323846
#endif
//======================

class Mask {
	
	public:

        // Constructor
        Mask(int ray, int alphaEdgeSize, int imageXSize, int imageYSize);
		
        // Implementation doesn't exist :
/*      void change(int ray, int alphaEdgeSize);
        void changeInnerArea(int ray);
        void changeBlendArea(int alphaEdgeSize); */

		~Mask();	
	
	protected:
		
        int xSize;                      // Destination image x size
        int ySize;                      // Destination image y size
        int innerRad;                   // Inner circle radius
        int outerRad;                   // Outer circle radius =
                                        // Inner circle + blend ring
                                        //  radius

        int alphaPixelsAmount;          // Blend area pixels count

        struct alphaPixel {             // Single blend pixel
            int x;                      // X position
            int y;                      // Y position
            float value;                // Alpha intensity
        };

        struct alphaPixel* blendPixels; // Blend area pixels table

        signed int *outerSpanLeft;      // Outer left edge table
        signed int *outerSpanRight;     // Outer right edge table

        signed int *innerSpanLeft;      // Inner left edge table
        signed int *innerSpanRight;     // Inner right edge table

        friend class Image8Bit;         // Only current user

        // Implementation doesn't exist
        // friend class Image16Bit;
};

#endif


